<?php
class FormValidation {
  private $rules;
  private $messages;
  private $isInvalid;
  private $invalidMessages;
  
  public function __construct($validationRules, $messages) {
    if(is_array($validationRules))
      $this->rules = $validationRules;
    else
      $this->rules = array();
      
    if(is_array($messages))
      $this->messages = $messages;
    else
      $this->messages = array();
    
    $fields = array_keys($this->rules);
    $this->isInvalid = array();
    $this->invalidMessages = array();
    foreach($fields as $field) {
      $this->isInvalid[$field] = 0;
      $this->invalidMessages[$field] = "";
    }
  }
  
  public function __get($name) {
    if (array_key_exists($name, get_class_vars(get_class())))
      return $this->$name;
      
    return null;
  }
  
  public function hasRules() {
    return (count($this->rules)>0);
  }
  
  public function isValid($field="") {
    if ($field=="")
      return !in_array(1, $this->isInvalid);
    else if(!isset($this->isInvalid[$field]))
      return 1;
    else
      return (!$this->isInvalid[$field]);
  }
  
  public function addInvalidField($field, $message) {
    $this->isInvalid[$field] = 1;
    $this->invalidMessages[$field] = $message;
  }
  
  public function getInvalidMessage($field) {
    if(!isset($this->invalidMessages[$field]))
      return "";
    else
      return $this->invalidMessages[$field];
  }
  
  public function getInvalidMessages() {
    return array_filter($this->invalidMessages);
  }
  
  public function addRules(FormValidation $newRules) {
    $this->rules = array_merge($this->rules, $newRules->rules);
    $this->messages = array_merge($this->messages, $newRules->messages);
  }
  
  private function getMessage($field, $rule, $value) {
    if (!isset($this->messages[$field]))
      return _("Value not allowed.");
    else if (is_array($this->messages[$field]))
      $message = $this->messages[$field][$rule];
    else
      $message = $this->messages[$field];
    
    if(strstr($message, "jQuery.validator.format")) {
      $message = preg_replace('~^jQuery\.validator\.format\("(.*)"\)$~', '\1', $message);
      $message = str_replace('{0}', $value, $message);
    }
    return $message;
  }
  
  private function getArrayValue($array, $fieldStr) {
    if(strpos($fieldStr, '[')!==false) {
      $fields = str_replace(']', '', explode("[", $fieldStr));
      $value = $array;
      foreach ($fields as $field) {
        if(!isset($value[$field])) {
          $value = null;
          break;
        }
        $value = $value[$field];
      }
      return $value;
    }
    else if(isset($array[$fieldStr]))
      return $array[$fieldStr];
    else 
      return null;
  }
  
  public function validate($array) {
    foreach ($this->rules as $field => $rule) {
      $this->isInvalid[$field] = 0;
      if(in_array("required", array_keys($rule)) && (is_null($this->getArrayValue($array, $field)) || $this->getArrayValue($array, $field)=="")) {
        $this->isInvalid[$field] = 1;
        $this->invalidMessages[$field] = $this->getMessage($field, "required", $this->getArrayValue($array, $field));
      }
      else if($error=$this->validateFormInput($field, $this->getArrayValue($array, $field), $array)) {
        $this->isInvalid[$field] = 1;
        $this->invalidMessages[$field] = $this->getMessage($field, $error, $this->getArrayValue($array, $field));
      }
    }
    return !in_array(1, $this->isInvalid);
  }
  
  private function validateFormInput($field, $value, $array) {
    $rule = $this->rules[$field];
    if(in_array("required", array_keys($rule)) && ($value=='' || $value==null))
      return "required";

    if((!isset($value) || !$value))
      return 0;
      
    foreach ($rule as $rule_key=>$rule_value) {
      switch ($rule_key) {
        case "url":
          break;
        case "email":
          if(!preg_match("~[a-z0-9!#$%&'*+/=?^_`{|}\~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}\~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?~",
                        $value))
            return $rule_key;
          break;
        case "remote":
          if(strpos($rule_value['url'], "admin/checkValue.php")!==false) {
            $class = $rule_value['data']['class'];
            $id = (!isset($array[substr($rule_value['data']['id'], 1)]) ? -1 : $array[substr($rule_value['data']['id'], 1)]);
            if($class::keyValueExists($value, $id)) {
              return $rule_key;
            }
          }
          break;
        case "date_de_DE":
          if(!preg_match("~^[0-3]?\d\.[0-1]?\d\.(\d{2}|\d{4})$~", $value))
            return $rule_key;
          break;
        case "date_en_US":
          if(!preg_match("~^[0-1]?\d\/[0-3]?\d\/\d{2,4}$~", $value))
            return $rule_key;
          break;
        case "digits":
          if(!preg_match("~^[0-9]+$~", $value))
            return $rule_key;
          break;
        case "number_int":
          if(!preg_match("~^[0-9]*([,.][0-9]*)?$~", $value))
            return $rule_key;
          break;
        case "equalTo":
          if (substr($rule_value, 0, 1)=="#") {
            if ($value!=$this->getArrayValue($array, substr($rule_value, 1)))
              return $rule_key;
          }
          else {
            if($value!=$rule_value)
              return $rule_key;
          }
          break;
      }
    }

    return 0;
  }

  public function getValidationRulesJQuery($indent="") {
    $txt = "{$indent}rules: {\n";
    
    foreach ($this->rules as $element=>$rule) {
      $txt .= "{$indent}  \"{$element}\": {\n";
      $ruleArray = array();
      
      foreach ($rule as $ruleName=>$value) {
        switch($ruleName) {
          case "remote":
            if(is_array($value['data'])) {
              $data = array();
              foreach ($value['data'] as $dataName => $dataPoint) {
                if(substr($dataPoint, 0, 1)=="#")
                  $data[] = "{$dataName}: function() { return \$('{$dataPoint}').val(); }";
                else 
                  $data[] = "{$dataName}: '{$dataPoint}'";
              }
              $data = implode(",\n                        ", $data);
            }
            else
              $data = "{$value['data']}: function() { return \$('#{$value['data']}').val(); }";
            $ruleArray[] = "    {$ruleName}: {\n".
                   $indent."        url: \"{$value['url']}\",\n".
                   $indent."        type: \"post\",\n".
                   $indent."        data: { {$data} }\n".
                   $indent."      }";
            break;
          default:
            switch(gettype($value)) {
              case "boolean":
                $ruleArray[] = "    {$ruleName}: ".(($value) ? "true" : "false");
                break;
              default:
                $ruleArray[] = "    {$ruleName}: \"{$value}\"";
            }
        }
      }
      $txt .= $indent.implode(",\n{$indent}", $ruleArray)."\n";
      $txt .= "$indent  },\n";
    }
    $txt .= "{$indent}},\n";
    $txt .= "{$indent}messages: {\n";
    foreach ($this->messages as $element=>$message) {
      if(!is_array($message))
        $txt .= "{$indent}  \"{$element}\": ".((substr($message, 0, 6)=="jQuery") ? $message : "\"{$message}\"").",\n";
      else {
        $txt .= "{$indent}  \"{$element}\": {\n";
        $msgArray = array();
        foreach ($message as $ruleName=>$ruleMessage)
          $msgArray[] = "    {$ruleName}: ".((substr($ruleMessage, 0, 6)=="jQuery") ? $ruleMessage : "\"{$ruleMessage}\"");
        $txt .= $indent.implode(",\n{$indent}", $msgArray)."\n";
        $txt .= "{$indent}  },\n";
      }
    }
    if(count($this->messages)>0) $txt = substr($txt, 0, -2);
    $txt .= "\n{$indent}}";
    return $txt;
  }
}
?>